@@ -261,6 +261,40 @@ gimp_main (const GimpPlugInInfo *info,
     if (p_SetDllDirectoryA)
       (*p_SetDllDirectoryA) ("");
   }
+
+  /* On Windows, set DLL search path to $INSTALLDIR/bin so that GEGL
+     file operations can find their respective file library DLLs (such
+     as jasper, etc.) without needing to set external PATH. */
+  {
+    const gchar *install_dir;
+    gchar       *bin_dir;
+    LPWSTR       w_bin_dir;
+    int          n;
+
+    w_bin_dir = NULL;
+    install_dir = gimp_installation_directory ();
+    bin_dir = g_build_filename (install_dir, "bin", NULL);
+
+    n = MultiByteToWideChar (CP_UTF8, MB_ERR_INVALID_CHARS,
+                             bin_dir, -1, NULL, 0);
+    if (n == 0)
+      goto out;
+
+    w_bin_dir = g_malloc_n (n + 1, sizeof (wchar_t));
+    n = MultiByteToWideChar (CP_UTF8, MB_ERR_INVALID_CHARS,
+                             bin_dir, -1,
+                             w_bin_dir, (n + 1) * sizeof (wchar_t));
+    if (n == 0)
+      goto out;
+
+    SetDllDirectoryW (w_bin_dir);
+
+  out:
+    if (w_bin_dir)
+      g_free (w_bin_dir);
+    g_free (bin_dir);
+  }
+
 #ifndef _WIN64
   {
     typedef BOOL (WINAPI *t_SetProcessDEPPolicy) (DWORD dwFlags);
@@ -401,8 +435,13 @@ gimp_main (const GimpPlugInInfo *info,
   gimp_signal_private (SIGCHLD, SIG_DFL, SA_RESTART);
 #endif
 
+#ifdef G_OS_WIN32
+  _readchannel  = g_io_channel_win32_new_fd (atoi (argv[2]));
+  _writechannel = g_io_channel_win32_new_fd (atoi (argv[3]));
+#else
   _readchannel  = g_io_channel_unix_new (atoi (argv[2]));
   _writechannel = g_io_channel_unix_new (atoi (argv[3]));
+#endif
 
   g_io_channel_set_encoding (_readchannel, NULL, NULL);
   g_io_channel_set_encoding (_writechannel, NULL, NULL);
